/* 	$Id: id3tag.c,v 1.1.1.1 2000/01/30 16:39:11 benediktroth Exp $	 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <glib.h>

#include <config.h>
#include "yamt.h"
#include "id3tag.h"

/* Field  Length  pos in tag  pos from end of file
 *  Tag      3       0-  2        -128
 *  Title   30       3- 32        -125
 *  Artist  30      33- 62         -95
 *  Album   30      63- 92         -65
 *  Year     4      93- 96         -35
 *  Comment 30      97-126         -31
 *  Genre    1         127          -1
 */

char *id3tag_genres_sorted[GENRE_MAX+1] = {NULL};


int id3tag_initialize( struct id3tag *tag )
{
  strcpy( tag->tag, "TAG" );
  tag->title[0] = '\0';
  tag->artist[0] = '\0';
  tag->album[0] = '\0';
  tag->comment[0] = '\0';
  tag->year[0] = '\0';
  tag->genre = 0xFF;
  /*   strcpy( tag->artist,  "                             " ); */
  /*   strcpy( tag->album,   "                             " ); */
  /*   strcpy( tag->comment, "                             " ); */
  /*   strcpy( tag->year,    "    " );   */
  return(1);
}

/* Reads the tag of a file and returns it in the id3tag structure */
struct id3tag id3tag_get( char *filename )
{
  struct id3tag         tag; 
  FILE                  *file;

  strcpy( tag.tag, "FOO" );
  
  if ( (file=fopen(filename, "r")) == 0)
    {
      printf("\n");
      printf("id3tag_get: ERROR while opening %s!\n", filename);
      return(tag);
    }

  /* Get the ID3 TAG */
  fseek(file, -128, SEEK_END);

  fread( &(tag.tag),      1,  3, file);

  if( strncmp(tag.tag, "TAG", 3) !=0 )
    {
      printf("WARNING: %s has no ID3 TAG!\n", filename);
      strcpy( tag.tag, "FOO" );
      return(tag);
    }

  fread( &(tag.title),    1, 30, file);  tag.title[30]='\0';
  fread( &(tag.artist),   1, 30, file);  tag.artist[30]='\0';
  fread( &(tag.album),    1, 30, file);  tag.album[30]='\0';
  fread( &(tag.year),     1,  4, file);  tag.year[4]='\0';
  fread( &(tag.comment),  1, 30, file);  tag.comment[30]='\0';
  fread( &(tag.genre),    1,  1, file);

  /* Cut off spaces at the end */
  id3tag_cut( tag.title );
  id3tag_cut( tag.artist );
  id3tag_cut( tag.album );
  id3tag_cut( tag.year );
  id3tag_cut( tag.comment );
  
  fclose(file);
  return(tag);
}

/* Return a tag created from a filename */
/* Usefull for files in the 'foo - bar.mp3' format */
/* What fields should be used for the two parts must be specified in part1 and part2 */
/* mptc means MoveParenthesesToComment */
struct id3tag id3tag_get_from_filename( char *filename, char *part1, char *part2, char separator, int mptc )
{
  struct id3tag tag;
  char buffer[30]="";
  char *tmp;
  char *end;
  char *ptr;
  int i;
  int i2;
  int i3;

  printf("\n");
  
  id3tag_initialize( &tag );
  /* Make the tag invalid */
  strcpy( tag.tag, "FOO" );

  /* Only the filename without the path */
  filename = strdup(g_basename(filename));
  
  /* Cut off the .mp3 */
  tmp = strrchr( filename, '.' );
  if( tmp != NULL )
    *tmp=0;

  /* If the separator cannot be found in the filename */
  if( (tmp=strchr( filename, separator )) == NULL )
    return(tag);

  /* Distance beetween the beginning of the filename and the separator */
  end = tmp-filename;

  /* The first part until the separator */
  for( i=0; i<end && i<30; i++ )
      buffer[i] = filename[i];

  id3tag_fill( buffer, 30 );
     
  /* See where the text should go */
  if( !strcmp(part1, "Title") )
    strncpy( tag.title, buffer, 30 );
  else if ( !strcmp(part1, "Artist") )
    strncpy( tag.artist, buffer, 30 );
  else if( !strcmp(part1, "Album") )
    strncpy( tag.album, buffer, 30 );
  else if( !strcmp(part1, "Comment") )
    strncpy( tag.comment, buffer, 30 );
  else if( !strcmp(part1, "Year") )
    strncpy( tag.year, buffer, 4 );
  else if( !strcmp(part1, "Genre") )
    {
      id3tag_cut( buffer );
      /* id3tag_lookup_genre() is case sensitive! */
      buffer[0] = toupper(buffer[0]);
      tag.genre = id3tag_lookup_genre( buffer );
    }
  else
    return(tag);

  /* jump over the seperator and spaces*/
  while( isspace(filename[i]) || filename[i]==separator)
    i++;

  /* Move everything between parantheses (I hate this word) into the comment field */
  if( mptc )
    {
      if( (ptr=strchr(filename,'(')) != NULL )
	{
 	  end = ptr-filename;
	  
	  for( i3=ptr-filename, i2=0; filename[i3] != '0' && i<30; i3++, i2++ )
	    buffer[i2] = filename[i3];

	  id3tag_fill( buffer, 30 );
	  strncpy( tag.comment, buffer, 30 );
	}
    }
  else
    end = 30;

  /* Now the rest behind the separator */
  for( i2=0; i<end && filename[i] != '\0'; i++, i2++)
    buffer[i2] = filename[i];
  buffer[i2]='\0';
  
  /* If there are chars left put them into the comment field */
  if( filename[i] != '\0')
    {
      if( (ptr=strrchr(buffer,'(')) != NULL )
	{
	  strcpy(tag.comment,ptr);
	  printf("tag.comment=%s\n",tag.comment);
	  *ptr='\0';
	  i-=strlen(tag.comment);
	}
      for( i2=0; i2<30 && filename[i] != '\0'; i++, i2++)
	tag.comment[i2] = filename[i];
      tag.comment[i2]='\0';
    }
   
  g_free( filename );

  /* See where the text should go */
  if( !strcmp(part2, "Title") )
    strncpy( tag.title, buffer, 30 );
  else if ( !strcmp(part2, "Artist") )
    strncpy( tag.artist, buffer, 30 );
  else if( !strcmp(part2, "Album") )
    strncpy( tag.album, buffer, 30 );
  else if( !strcmp(part2, "Comment") )
    strncpy( tag.comment, buffer, 30 );
  else if( !strcmp(part2, "Year") )
    strncpy( tag.year, buffer, 4 );
  else if( !strcmp(part2, "Genre") )
    {
      id3tag_cut( buffer );
      /* id3tag_lookup_genre() is case sensitive! */
      buffer[0] = toupper(buffer[0]);
      tag.genre = id3tag_lookup_genre( buffer );
    }
  else
    return(tag);


  id3tag_fill( tag.album, 30 );
  id3tag_fill( tag.comment, 30 );
  id3tag_fill( tag.year, 4 );
				  
  /* Make the tag valid */
  strcpy( tag.tag, "TAG" );
  return( tag );
}

/* Shows a tag */
int id3tag_show( struct id3tag tag )
{
  printf("Title:    %s\n", tag.title); 
  printf("Artist:   %s\n", tag.artist); 
  printf("Album:    %s\n", tag.album);
  printf("Year:     %s \t\tGenre: %s\n", tag.year, id3tag_get_genre(tag.genre) );
  printf("Comment:  %s\n", tag.comment);
  return(1);
}

/* This function just appends an empty tag to the file */
int id3tag_new( char *filename )
{
  FILE *file;
  struct id3tag tag;

  
  if ( (file=fopen(filename, "a")) == 0)
    {
      printf("\n");
      printf("id3tag_new: ERROR while opening %s!\n", filename);
      return(-1);
    }

  strcpy( tag.title,   "                             " );
  strcpy( tag.artist,  "                             " );
  strcpy( tag.album,   "                             " );
  strcpy( tag.comment, "                             " );
  strcpy( tag.year,    "    " );
  tag.genre=0xFF;

  printf("Writing a new ID3 TAG to %s...\n", filename);
  /* Write the ID3 TAG */
  fseek( file, 0, SEEK_END );
  
  fwrite( (tag.tag)     ,   1,   3, file);
  fwrite( (tag.title)   ,   1,  30, file);
  fwrite( (tag.artist)  ,   1,  30, file);
  fwrite( (tag.album)   ,   1,  30, file);
  fwrite( (tag.comment) ,   1,  30, file);
  fwrite( (tag.year)    ,   1,   4, file);

  fwrite( &(tag.genre)  ,   1,   1, file);

  fclose(file);

  return(1);
}

/* Removes a tag */
int id3tag_remove( char *filename )
{
  int fd, len;

  printf("Removing ID3 TAG in %s\n", filename );
  
  if ((fd = open(filename, O_RDWR)) != -1)
    {
      len = lseek(fd, -128, SEEK_END);

      /* Reduce the lenght of the file */
      if (ftruncate(fd, len))
	{
	  printf("Couldn't remove TAG\n");
	  return(-1);
	}
      printf("ok!\n");
      close(fd);
      return(1);
    }
  else
    return(0);
}

/* Writes the given tag to the file if it does not already contain a tag */
/* id3tag_new() is called to create one, then teh given tag will be set */
/* Only fields which beginning is != '\0' will be written ! */
/* This is usefull for the mass tagging so that only the data will be written */
/* that should be written */
int id3tag_set( char *filename, struct id3tag tag )
{
  FILE *file;
  char buffer[3];

  if ( (file=fopen(filename, "r+")) == 0)
    {
      g_warning("\nid3tag_set: ERROR while opening %s!\n", filename);
      return(-1);
    }

  /* If the tag that should be written is not valid do not write it */
  if( !id3tag_is_valid(tag) )
    {
      printf("WARNING: id3tag.set: No valid tag! Skipping %s\n", filename);
      return(-1);
    }

  /* Check if the file has already a tag, if not create one */
  fseek(file, -128, SEEK_END);
  fread( &(buffer),      1,  3, file);
  if( strncmp(buffer, "TAG", 3) !=0 )
    {
      printf("WARNING: %s has no ID3 TAG!\n", filename);
      fclose( file );
      
      id3tag_new( filename );

      if ( (file=fopen(filename, "r+")) == 0)
	{
	  g_warning("\nid3tag_set: ERROR while opening %s!\n", filename);
	  return(-1);
	}
    }  

  
  /* Write the ID3 TAG */
  fseek(file, -128, SEEK_END);
  fwrite( (tag.tag),      1,  3, file);
  
  if( tag.title[0] != '\0' )
    {
      fseek( file, -125, SEEK_END); 
      id3tag_fill( tag.title, 30 );
      fwrite( (tag.title),    1, 30, file);
    }

  if( tag.artist[0] != '\0' )
    {
      fseek( file, -95, SEEK_END); 
      id3tag_fill( tag.artist, 30 );
      fwrite( (tag.artist),   1, 30, file);
    }
  
  if( tag.album[0] != '\0' )
    {
      fseek( file, -65, SEEK_END); 
      id3tag_fill( tag.album, 30 );
      fwrite( (tag.album),    1, 30, file);
    }

  if( tag.year[0] != '\0' )
    {
      fseek( file, -35, SEEK_END); 
      id3tag_fill( tag.year, 4 );
      fwrite( (tag.year),     1,  4, file);
    }

  if( tag.comment[0] != '\0' )
    {
      fseek( file, -31, SEEK_END); 
      id3tag_fill( tag.comment, 30 );
      fwrite( (tag.comment),  1, 30, file);
    }

  if( tag.genre != 0xFF )
    {
      fseek( file, -1, SEEK_END); 
      fwrite( &(tag.genre),     1,  1, file);
    }

  fclose(file);
  return(1);
}

/* This function renames a file based on its tag in the given format */
int id3tag_rename( char *filename, char *format )
{
  struct id3tag tag;
  struct stat stbuf;
  char target_filename[80]="";
  char buffer[10]="";
  char *tmp;
  int i;
  int i2;

  
  tag=id3tag_get( filename );

  /* If the file has no valid tag */
  if(strncmp(tag.tag, "TAG", 3) != 0 )
    {
      yamtlog("%s %s %s", "id3tag_rename(): Skipped ", filename, "No valid tag!");
      return(0);
    }
  
  /* The spaces at the end of the strings must be removed */
  id3tag_cut( tag.title );  
  id3tag_cut( tag.artist );
  id3tag_cut( tag.album );
  id3tag_cut( tag.comment );
   
  /* See what the user wants the format to be */
  for(i=0, i2=0; i<strlen(format); i++)
    {
      if( format[i] == '%' && format[i+1] != '%' && format[i-1] != '%')
	{
	  switch (format[i+1] )
	    {
	    case 't':
	      strcat( target_filename, tag.title);
	      i2=i2+strlen(tag.title);
	      i++;
	      break;
	      
	    case 'a':
	      strcat( target_filename, tag.artist);
	      i2=i2+strlen(tag.artist);
	      i++;	      
	      break;

	    case 'b':
	      strcat( target_filename, tag.album);
	      i2=i2+strlen(tag.album);
	      i++;	      
	      break;

	    case 'c':
	      strcat( target_filename, tag.comment);
	      i2=i2+strlen(tag.comment);
	      i++;	      
	      break;
		
	    case 'y':
	      strcat( target_filename, tag.year);
	      i2=i2+strlen(tag.year);
	      i++;	      
	      break;

	    case 'g':
	      strcat( target_filename, id3tag_get_genre(tag.genre));
	      i2=i2+strlen(id3tag_get_genre(tag.genre));	      
	      i++;
	      break;
		
	    default:
	      printf("Format Argument not found: %c !\n", format[i+1] );
	      return(-1);
	      break;
		
	    } /* End switch */
	} /* End if */
      else
	{
	  target_filename[i2]=format[i];
	  i2++;
	}
    } /* End for */
		
/*   id3tag_show( tag ); */

  /* Use .mp3 as the file extension */
  strcat( target_filename, ".mp3" );
  
/*   printf("target_filename: %s\n", target_filename); */

  /* If the file doesn't need to be moved (already named correctly) */
  if ( strcmp( filename, target_filename) ==0 )
    {
      yamtlog("%s %s %s", "File ", filename, " was already named correctly");
      return(1);
    }

  for( i=1; ;i++)
    {
      /* If the file existst */
      if( stat( target_filename, &stbuf ) == 0)
	{
/* 	  printf("id3tag_rename: ERROR: File \"%s\" exists already!\n", target_filename); */

	  /* Cut off the previously created numbers */
	  /* so we don't have something like "foo - bar #1 #2 #3.mp3" */
	  target_filename[strlen(target_filename) - strlen(buffer) - 4] = '\0';
	  
	  sprintf( buffer, " #%i", i);
	  strcat( target_filename, buffer );
	  strcat( target_filename, ".mp3" );
	}
      else
	break;
    }
  
  if( (rename( filename, target_filename )) )  
    {  
      yamtlog("%s %s %s %s", "id3tag_rename(): ERROR while renaming ", filename, " to ", target_filename); 
      return(-1);  
    } 
  yamtlog("%s %s %s %s", "File ", filename, " moved to ",target_filename);
  
  return(1);
}

/* Sorts mp3s */
/* Creates subdirs in the given format and moves the file into them */
int id3tag_sort( char *filename, char *rootdir, char *format_level1, char *format_level2 )
{
  struct id3tag tag;
  char *dir_level1=NULL;
  char *dir_level2=NULL;
  char target_filename[80];
  char dir_cur[80];

  
  getcwd( dir_cur, sizeof(dir_cur) );
  tag = id3tag_get( filename );
  /* Check if the file has a valid ID3 TAG */
  if( strncmp(tag.tag,"TAG",3) != 0 )
    {
      yamtlog("%s %s", "id3tag_sort(): ", filename, " not sorted!");
      return(0);
    }
  
  /* Level 1 */
  if( strcmp( format_level1, "Artist") == 0 )
    dir_level1 = tag.artist;
  else if( strcmp( format_level1, "Album") == 0 )
    dir_level1 = tag.album;
  else if( strcmp( format_level1, "Year") == 0 )
    dir_level1 = tag.year;
  else if( strcmp( format_level1, "Genre") == 0 )
    dir_level1 = id3tag_get_genre(tag.genre);
  else if( strcmp( format_level1, "Comment") == 0 )
    dir_level1 = tag.comment;
  else if( strcmp( format_level1, "Title") == 0 )
    dir_level1 = tag.title;
      
  /* Go to the root directory */
  chdir(rootdir);
  if( dir_level1[0] == '\0' )
    dir_level1 = "Unknown";
  yamtlog("%s %s", "New directory: ", dir_level1);
  mkdir( dir_level1, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH );
	  
  /* Level 2 */
  if( strcmp( format_level2, "Album") == 0 )
    dir_level2 = tag.album;
  else if( strcmp( format_level2, "Artist") == 0 )
    dir_level2 = tag.artist;
  else if( strcmp( format_level2, "Year") == 0 )
    dir_level2 = tag.year;
  else if( strcmp( format_level2, "Genre") == 0 )
    dir_level2 = id3tag_get_genre(tag.genre);
  else if( strcmp( format_level2, "Comment") == 0 )
    dir_level2 = tag.comment;
  else if( strcmp( format_level2, "Title") == 0 )
    dir_level2 = tag.title;
  
  if( dir_level2[0] == '\0' )
    dir_level2 = "Unknown";
  yamtlog("%s %s", "New directory: ", dir_level2);

  /* Go into the previously created directory */
  chdir( dir_level1 );
  mkdir( dir_level2, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH );
  /* Move the file into the new (?) directory */

  sprintf( target_filename, "mv \"%s/%s\" \"%s%s/%s/%s\"", dir_cur, filename, rootdir, dir_level1, dir_level2, filename ); 

  yamtlog("%s %s", "Sorted ", filename );

  system( target_filename ); 
  
/*   if( (rename( filename, target_filename )) )   */
/*     {   */
/*       printf("id3tag.c, id3tag_sort: ERROR while renaming %s to %s!\n", filename, target_filename);  */
/*       return(-1);   */
/*     }    */
  return(1);
}

/* Compares to tags */
/* mode 0 = no exact match */
/* mode 1 = exact match    */
/* 0 means success!        */
int id3tag_cmp( struct id3tag tag, struct id3tag tag2, int mode )
{
  int   result=0;
  char *chpt="X";
  int   no=0;     /* Will be set to 1 if a comparison failed */

  if( mode == 0 )
    {
      /* the search should not be case sensitive */
      /* I wonder if this should be optional */
      /* I don't need it */
            
      if( tag.title[0] != '\0' && no == 0 )
	{
	  id3tag_lower( tag.title );
	  id3tag_lower( tag2.title );
	  chpt = strstr( tag2.title, tag.title);
	  if( chpt == 0 )
	    no = 1;
	}

      if( tag.artist[0] != '\0' && no == 0 )
	{
	  id3tag_lower( tag.artist );
	  id3tag_lower( tag2.artist );
	  chpt = strstr( tag2.artist, tag. artist);
	  if( chpt == 0 )
	    no = 1;
	}

      if( tag.album[0] != '\0' && no == 0 )
	{
	  id3tag_lower( tag.album );
	  id3tag_lower( tag2.album );
	  chpt = strstr( tag2.album, tag.album );
	  if( chpt == 0 )
	    no = 1;
	}
  
      if( tag.comment[0] != '\0' && no == 0 )
	{
	  id3tag_lower( tag.comment );
	  id3tag_lower( tag2.comment );
	  chpt = strstr( tag2.comment, tag.comment );
	  if( chpt == 0 )
	    no = 1;
	}
  
      if( tag.year[0] != '\0' && no == 0 )
	{
	  id3tag_lower( tag.year );
	  id3tag_lower( tag2.year );
	  chpt = strstr( tag2.year, tag.year );
	  if( chpt == 0 )
	    no = 1;
	}
  
      if( tag.genre >= 0 && no == 0 ) 
	{ 
	  if( tag2.genre == tag.genre)
	    chpt = "X";
	  else
	    chpt = 0;
	  
	  if( chpt == 0 )
	    no = 1;
	}

      if( no == 1 )
	chpt = 0;

      
      if( chpt != 0 && no == 0 )
	return(0);
      else
	return(1);
    }
  else if( mode == 1)
    {
      if( tag.title[0] != '\0' && no == 0 )
	{
	  result = strcmp( tag2.title, tag.title);
	  if( result != 0 )
	    no = 1;
	}
   
      if( tag.artist[0] != '\0' && no == 0 )
	{
	  result = strcmp( tag2.artist, tag. artist);
	  if( result != 0 )
	    no = 1;
	}
  
      if( tag.album[0] != '\0' && no == 0 )
	{
	  result = strcmp( tag2.album, tag.album );
	  if( result != 0 )
	    no = 1;
	}
  
      if( tag.comment[0] != '\0' && no == 0 )
	{
	  result = strcmp( tag2.comment, tag.comment );
	  if( result != 0 )
	    no = 1;
	}
  
      if( tag.year[0] != '\0' && no == 0 )
	{
	  result = strcmp( tag2.year, tag.year );
	  if( result != 0 )
	    no = 1;
	}
  
      if( tag.genre >= 0 && no == 0 ) 
	{ 
	  if( tag2.genre == tag.genre)
	    result = 0;
	  else
	    result = 1;

	  if( result != 0 )
	no = 1;
	}
      
      if( no == 1 )
	result = 1;

      
      return(result);
    }
  else
    return(1);
}


/* This function is used to cut the spaces at the end of an string */
/* Background: The fields must be exactly 30 (title, artist, album and comment) */
/* or 4 (year) characters long, so they must be filled to this size */
/* when they should be written, but after reading the should be cutted */
void id3tag_cut( char *string )
{
  int i;

  /* Toshiaki Kanosue 2000/1/21 for Japanese Kanji */
  for( i=strlen(string); !(isgraph(string[i])) && !(string[i] & 0x80); i-- )
    if( i < 0 )
      break;
  
  string[++i]='\0';
}

/* This function adds spaces to the end of a string to the given size (end) */
/* Background: see above */
void id3tag_fill( char *string, int end )
{
  int i;

  for( i=strlen(string); i<end ; i++ )
    string[i]=' ';

  string[end]='\0';
}

/* tolower's every character in a string */
void id3tag_lower( char *string )
{
  int i;

  for( i=0; string[i]!='\0'; i++ )
    string[i] = tolower(string[i] );
}

/* Sorts the genres in id3tag_genre_names[] to id3tag_genres_sorted */
/* So that it can be used to create the pull down menu */
void id3tag_sort_genres(void)
{
    int i, j;
    char *tmp;

    if (id3tag_genres_sorted[0] != NULL)
	return;

    for (i=0; i<GENRE_MAX; i++)
	id3tag_genres_sorted[i] = id3tag_genre_names[i];

    /* FIXME: did I miss something here, or is qsorting an array of strings not possible? */
    /* qsort(id3tag_genres_sorted, GENRE_MAX, sizeof(*id3tag_genres_sorted), strcmp); */

    /* bubble sort for now ... */
    for (j=GENRE_MAX-1; j>0; j--)
	for (i=0; i<j; i++)
	    if (strcmp(id3tag_genres_sorted[i], id3tag_genres_sorted[i+1]) > 0) {
		tmp = id3tag_genres_sorted[i];
		id3tag_genres_sorted[i] = id3tag_genres_sorted[i+1];
		id3tag_genres_sorted[i+1] = tmp;
	    }
    id3tag_genres_sorted[GENRE_MAX] = "";
}

/* Returns the name of a genre code */
const char *id3tag_get_genre(const unsigned char genre_code)
{
    g_assert(genre_code >= 0);
    g_assert(genre_code <= 0xFF);
    if (genre_code == 0xFF) { /* empty */
	return "";
    } else if (genre_code >= GENRE_MAX) { /* unknown tag */
	return "Unkown!";
    } else { /* known tag */
	return id3tag_genre_names[genre_code];
    }
}

/* Return the genre code of a given name */
const char id3tag_lookup_genre(const char *genre)
{
    int i;
    for (i=0; i<GENRE_MAX; i++)
	if (!strcmp(genre, id3tag_genre_names[i]))
	    return i;
    if (genre[0]) g_warning("id3tag_lookup: unkown genre \"%s\"!\n", genre);
    return 0xFF;
}

/* Checks if a tag is valid */
int id3tag_is_valid( struct id3tag tag )
{
  if( strncmp(tag.tag, "TAG", 3) !=0 )
    return(0);
  else  
    return(1);
}
